@@ -1,16 +1,15 @@
 BUILD_BIT='64bit'
 BUILD_CPU='x86_64'
-BUILD_DATE='2020-06-05'
+BUILD_DATE='2020-08-08'
 BUILD_MACHINE='x86_64'
 BUILD_OS='Linux Fedora 31 ThirtyOne'
-BUILD_TIME='16:42'
+BUILD_TIME='13:51'
 BUILT_BY='antoine'
-BUILT_ON='desktop-eth0'
+BUILT_ON='localhost.localdomain'
 COMPILER_VERSION='gcc (GCC) 9.3.1 20200408 (Red Hat 9.3.1-2)'
 CYTHON_VERSION='3.0a4'
 LINKER_VERSION='GNU ld version 2.32-31.fc31'
-NVCC_VERSION='10.2.89'
-PYTHON_VERSION='3.7.7'
+PYTHON_VERSION='3.7.8'
 RELEASE_BUILD=True
 lib_gobject_introspection='1.62.0'
 lib_gtk='3.24.13'
@@ -22,6 +21,7 @@ lib_pygobject='3.34.0'
 lib_python3='3.7'
 lib_vpx='1.8.2'
 lib_x11='1.6.9'
+lib_x265='3.4'
 lib_xcomposite='0.4.4'
 lib_xdamage='1.1.4'
 lib_xext='1.3.4'
