@@ -9,6 +9,8 @@ import math
 import ctypes
 import struct
 import weakref
+
+from gi.repository import GLib
 import objc                         #@UnresolvedImport
 import Quartz                       #@UnresolvedImport
 import Quartz.CoreGraphics as CG    #@UnresolvedImport
@@ -32,9 +34,9 @@ from Foundation import (
     NSUserNotificationDefaultSoundName,             #@UnresolvedImport
     )
 
-from xpra.os_util import PYTHON2
 from xpra.util import envbool, envint, roundup
 from xpra.notifications.notifier_base import NotifierBase
+from xpra.platform.darwin import get_OSXApplication
 from xpra.log import Logger
 
 log = Logger("osx", "events")
@@ -44,9 +46,9 @@ notifylog = Logger("osx", "notify")
 
 OSX_FOCUS_WORKAROUND = envint("XPRA_OSX_FOCUS_WORKAROUND", 2000)
 SLEEP_HANDLER = envbool("XPRA_OSX_SLEEP_HANDLER", True)
-EVENT_LISTENER = envbool("XPRA_OSX_EVENT_LISTENER", True)
 OSX_WHEEL_MULTIPLIER = envint("XPRA_OSX_WHEEL_MULTIPLIER", 100)
 OSX_WHEEL_PRECISE_MULTIPLIER = envint("XPRA_OSX_WHEEL_PRECISE_MULTIPLIER", 1)
+OSX_WHEEL_DIVISOR = envint("XPRA_OSX_WHEEL_DIVISOR", 10)
 WHEEL = envbool("XPRA_WHEEL", True)
 NATIVE_NOTIFIER = envbool("XPRA_OSX_NATIVE_NOTIFIER", False)
 SUBPROCESS_NOTIFIER = envbool("XPRA_OSX_SUBPROCESS_NOTIFIER", False)
@@ -64,36 +66,11 @@ ALPHA = {
 try:
     Carbon_ctypes = ctypes.CDLL("/System/Library/Frameworks/Carbon.framework/Carbon")
     GetDblTime = Carbon_ctypes.GetDblTime
-except:
+except Exception:
+    log("GetDblTime not found", exc_info=True)
     GetDblTime = None
 
 
-exit_cb = None
-def quit_handler(*_args):
-    global exit_cb
-    if exit_cb:
-        exit_cb()
-    else:
-        from xpra.gtk_common.quit import gtk_main_quit_really
-        gtk_main_quit_really()
-    return True
-
-def set_exit_cb(ecb):
-    global exit_cb
-    exit_cb = ecb
-
-
-macapp = None
-def get_OSXApplication():
-    global macapp
-    if macapp is None:
-        from xpra.gtk_common.gobject_compat import import_gtkosx_application
-        gtkosx_application = import_gtkosx_application()
-        macapp = gtkosx_application.Application()
-        macapp.connect("NSApplicationWillTerminate", quit_handler)
-    return macapp
-
-
 def do_init():
     osxapp = get_OSXApplication()
     log("do_init() osxapp=%s", osxapp)
@@ -123,7 +100,7 @@ def do_ready():
 class OSX_Notifier(NotifierBase):
 
     def __init__(self, closed_cb=None, action_cb=None):
-        NotifierBase.__init__(self, closed_cb, action_cb)
+        super().__init__(closed_cb, action_cb)
         self.notifications = {}
         self.notification_center = NSUserNotificationCenter.defaultUserNotificationCenter()
         assert self.notification_center
@@ -464,24 +441,10 @@ def get_info():
 VIEW_TO_WINDOW = weakref.WeakValueDictionary()
 
 def add_window_hooks(window):
-    if WHEEL and PYTHON2:
-        global VIEW_TO_WINDOW
-        try:
-            gdkwin = window.get_window()
-            VIEW_TO_WINDOW[gdkwin.nsview] = window
-        except:
-            log("add_window_hooks(%s)", window, exc_info=True)
-            log.error("Error: failed to associate window %s with its nsview", window, exc_info=True)
+    pass
 
 def remove_window_hooks(window):
-    if WHEEL and PYTHON2:
-        global VIEW_TO_WINDOW
-        #this should be redundant as we use weak references:
-        try:
-            gdkwin = window.get_window()
-            del VIEW_TO_WINDOW[gdkwin.nsview]
-        except:
-            pass
+    pass
 
 
 def get_CG_imagewrapper(rect=None):
@@ -526,17 +489,7 @@ def take_screenshot():
 
 def force_focus(duration=2000):
     enable_focus_workaround()
-    from xpra.gtk_common.gobject_compat import import_glib
-    glib = import_glib()
-    glib.timeout_add(duration, disable_focus_workaround)
-
-
-def show_with_focus_workaround(show_cb):
-    from xpra.gtk_common.gobject_compat import import_glib
-    glib = import_glib()
-    enable_focus_workaround()
-    show_cb()
-    glib.timeout_add(500, disable_focus_workaround)
+    GLib.timeout_add(duration, disable_focus_workaround)
 
 
 __osx_open_signal = False
@@ -548,9 +501,7 @@ def add_open_handlers(open_file_cb, open_url_cb):
     assert open_file_cb and open_url_cb
 
     def idle_add(fn, *args):
-        from xpra.gtk_common.gobject_compat import import_glib
-        glib = import_glib()
-        glib.idle_add(fn, *args)
+        GLib.idle_add(fn, *args)
 
     def open_URL(url):
         global __osx_open_signal
@@ -570,8 +521,6 @@ def add_open_handlers(open_file_cb, open_url_cb):
 
 def wait_for_open_handlers(show_cb, open_file_cb, open_url_cb, delay=OPEN_SIGNAL_WAIT):
     assert show_cb and open_file_cb and open_url_cb
-    from xpra.gtk_common.gobject_compat import import_glib
-    glib = import_glib()
 
     add_open_handlers(open_file_cb, open_url_cb)
     def may_show():
@@ -580,7 +529,7 @@ def wait_for_open_handlers(show_cb, open_file_cb, open_url_cb, delay=OPEN_SIGNAL
         if not __osx_open_signal:
             force_focus()
             show_cb()
-    glib.timeout_add(delay, may_show)
+    GLib.timeout_add(delay, may_show)
 
 def register_file_handler(handler):
     log("register_file_handler(%s)", handler)
@@ -618,10 +567,6 @@ class Delegate(NSObject):
         log("applicationDidFinishLaunching_(%s)", notification)
         if SLEEP_HANDLER:
             self.register_sleep_handlers()
-        if WHEEL and PYTHON2:
-            from xpra.platform.darwin.gdk_bindings import init_quartz_filter, set_wheel_event_handler   #@UnresolvedImport
-            set_wheel_event_handler(self.wheel_event_handler)
-            init_quartz_filter()
 
     @objc.python_method
     def wheel_event_handler(self, nsview, deltax, deltay, precise):
@@ -637,7 +582,7 @@ class Delegate(NSObject):
                 m = OSX_WHEEL_PRECISE_MULTIPLIER
             else:
                 m = OSX_WHEEL_MULTIPLIER
-            v = abs(distance)/10.0*m
+            v = m*abs(distance)/OSX_WHEEL_DIVISOR
             if v>1:
                 #cancel out some of the crazy fast scroll acceleration from macos:
                 v = math.sqrt(v)
@@ -657,10 +602,10 @@ class Delegate(NSObject):
     @objc.python_method
     def register_sleep_handlers(self):
         log("register_sleep_handlers()")
-        self.workspace          = NSWorkspace.sharedWorkspace()
-        self.notificationCenter = self.workspace.notificationCenter()
+        workspace          = NSWorkspace.sharedWorkspace()
+        notificationCenter = workspace.notificationCenter()
         def add_observer(fn, val):
-            self.notificationCenter.addObserver_selector_name_object_(self, fn, val, None)
+            notificationCenter.addObserver_selector_name_object_(self, fn, val, None)
         #NSWorkspaceWillPowerOffNotification
         add_observer(self.receiveSleepNotification_, NSWorkspaceWillSleepNotification)
         add_observer(self.receiveWakeNotification_, NSWorkspaceDidWakeNotification)
@@ -737,7 +682,7 @@ def can_access_display() -> bool:
     return True
 
 
-class ClientExtras(object):
+class ClientExtras:
     def __init__(self, client=None, opts=None):
         if OSX_FOCUS_WORKAROUND and client:
             def first_ui_received(*_args):
@@ -776,11 +721,10 @@ class ClientExtras(object):
         self.client = None
 
     def ready(self):
-        if EVENT_LISTENER:
-            try:
-                self.setup_event_listener()
-            except:
-                log.error("Error setting up OSX event listener", exc_info=True)
+        try:
+            self.setup_event_listener()
+        except:
+            log.error("Error setting up OSX event listener", exc_info=True)
 
     def setup_event_listener(self):
         log("setup_event_listener()")
