@@ -1,21 +1,17 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
 import os
 
-#this is here so we can expose the python "platform" module
-#before we import xpra.platform
-import platform as python_platform
-assert python_platform
 from xpra.util import csv
 from xpra.os_util import (
-    WIN32, OSX, PYTHON2, POSIX,
+    WIN32, OSX, POSIX,
     osexpand, getuid, getgid, get_username_for_uid,
-    is_CentOS, is_RedHat, is_Fedora, is_Debian, is_Ubuntu, getUbuntuVersion,
+    is_Debian, is_arm,
     )
 
 def warn(msg):
@@ -33,7 +29,7 @@ class InitInfo(Exception):
 class InitExit(Exception):
     def __init__(self, status, msg):
         self.status = status
-        Exception.__init__(self, msg)
+        super().__init__(msg)
 
 
 DEBUG_CONFIG_PROPERTIES = os.environ.get("XPRA_DEBUG_CONFIG_PROPERTIES", "").split()
@@ -43,10 +39,10 @@ DEFAULT_NET_WM_NAME = os.environ.get("XPRA_NET_WM_NAME", "Xpra")
 
 if POSIX:
     DEFAULT_POSTSCRIPT_PRINTER = os.environ.get("XPRA_POSTSCRIPT_PRINTER", "drv:///sample.drv/generic.ppd")
-else:
+else: # pragma: no cover
     DEFAULT_POSTSCRIPT_PRINTER = ""
 DEFAULT_PULSEAUDIO = None   #auto
-if OSX or WIN32:
+if OSX or WIN32: # pragma: no cover
     DEFAULT_PULSEAUDIO = False
 
 
@@ -63,11 +59,8 @@ def has_sound_support():
 
 
 def get_xorg_bin():
-    xorg = os.environ.get("XPRA_XORG_BIN")
-    if xorg:
-        return xorg
     # Detect Xorg Binary
-    if os.uname()[4].startswith("arm") and is_Debian() and os.path.exists("/usr/bin/Xorg"):
+    if is_arm() and is_Debian() and os.path.exists("/usr/bin/Xorg"):
         #Raspbian breaks if we use a different binary..
         return "/usr/bin/Xorg"
     for p in (
@@ -80,7 +73,7 @@ def get_xorg_bin():
         if os.path.exists(p):
             return p
     #look for it in $PATH:
-    for x in os.environ.get("PATH").split(os.pathsep):
+    for x in os.environ.get("PATH").split(os.pathsep): # pragma: no cover
         xorg = os.path.join(x, "Xorg")
         if os.path.isfile(xorg):
             return xorg
@@ -92,7 +85,7 @@ def get_Xdummy_confdir():
     xrd = get_runtime_dir()
     if xrd:
         base = "${XDG_RUNTIME_DIR}/xpra"
-    else:
+    else:   # pragma: no cover
         base = "${HOME}/.xpra"
     return base+"/xorg.conf.d/$PID"
 
@@ -126,23 +119,20 @@ def get_Xvfb_command():
            ]
     return cmd
 
-def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None, Xdummy_ENABLED=None, Xdummy_wrapper_ENABLED=None):
+def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None,
+                        Xdummy_ENABLED=None, Xdummy_wrapper_ENABLED=None, warn=warn):
     #returns the xvfb command to use
-    if WIN32:
+    if WIN32:   # pragma: no cover
         return ""
-    if OSX:
+    if OSX:     # pragma: no cover
         return get_Xvfb_command()
-    if sys.platform.find("bsd")>=0 and Xdummy_ENABLED is None:
-        warn("Warning: sorry, no support for Xdummy on %s" % sys.platform)
+    if sys.platform.find("bsd")>=0 and Xdummy_ENABLED is None:  # pragma: no cover
+        if warn:
+            warn("Warning: sorry, no support for Xdummy on %s" % sys.platform)
         return get_Xvfb_command()
-    import platform
-    if platform.uname()[4].startswith("arm"):
+    if is_arm():
         #arm struggles to launch Xdummy, so use Xvfb:
         return get_Xvfb_command()
-    if is_Ubuntu() or is_Debian():
-        #These distros do weird things and this can cause the real X11 server to crash
-        #see ticket #2834
-        return get_Xvfb_command()
 
     xorg_bin = get_xorg_bin()
     def Xorg_suid_check():
@@ -150,7 +140,8 @@ def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None, Xdummy_ENABLED=None
             #honour what was specified:
             use_wrapper = Xdummy_wrapper_ENABLED
         elif not xorg_bin:
-            warn("Warning: Xorg binary not found, assuming the wrapper is needed!")
+            if warn:
+                warn("Warning: Xorg binary not found, assuming the wrapper is needed!")
             use_wrapper = True
         else:
             #auto-detect
@@ -158,7 +149,8 @@ def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None, Xdummy_ENABLED=None
             xorg_stat = os.stat(xorg_bin)
             if (xorg_stat.st_mode & stat.S_ISUID)!=0:
                 if (xorg_stat.st_mode & stat.S_IROTH)==0:
-                    warn("%s is suid and not readable, Xdummy support unavailable" % xorg_bin)
+                    if warn:
+                        warn("%s is suid and not readable, Xdummy support unavailable" % xorg_bin)
                     return get_Xvfb_command()
                 debug("%s is suid and readable, using the xpra_Xdummy wrapper" % xorg_bin)
                 use_wrapper = True
@@ -180,12 +172,6 @@ def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None, Xdummy_ENABLED=None
     if Xdummy_ENABLED is True:
         return Xorg_suid_check()
     debug("Xdummy support unspecified, will try to detect")
-
-    if is_Ubuntu():
-        rnum = getUbuntuVersion()
-        if rnum<(16, 10):
-            debug("Warning: Ubuntu Xenial breaks Xorg/Xdummy usage - using Xvfb fallback")
-            return get_Xvfb_command()
     return Xorg_suid_check()
 
 
@@ -194,24 +180,25 @@ def OpenGL_safety_check():
     #based on the code found here:
     #http://spth.virii.lu/eof2/articles/WarGame/vboxdetect.html
     #because it used to cause hard VM crashes when we probe the GL driver!
-    try:
-        from ctypes import cdll
-        if cdll.LoadLibrary("VBoxHook.dll"):
-            return "VirtualBox is present (VBoxHook.dll)"
-    except (ImportError, OSError):
-        pass
-    try:
+    if WIN32:   # pragma: no cover
         try:
-            f = None
-            f = open("\\\\.\\VBoxMiniRdrDN", "r")
-            return "VirtualBox is present (VBoxMiniRdrDN)"
-        finally:
-            if f:
-                f.close()
-    except Exception as e:
-        import errno
-        if e.args[0]==errno.EACCES:
-            return "VirtualBox is present (VBoxMiniRdrDN)"
+            from ctypes import cdll
+            if cdll.LoadLibrary("VBoxHook.dll"):
+                return "VirtualBox is present (VBoxHook.dll)"
+        except (ImportError, OSError):
+            pass
+        try:
+            try:
+                f = None
+                f = open("\\\\.\\VBoxMiniRdrDN", "r")
+                return True, "VirtualBox is present (VBoxMiniRdrDN)"
+            finally:
+                if f:
+                    f.close()
+        except Exception as e:
+            import errno
+            if e.args[0]==errno.EACCES:
+                return "VirtualBox is present (VBoxMiniRdrDN)"
     return None
 
 
@@ -285,11 +272,7 @@ def read_config(conf_file):
     if not os.path.isfile(conf_file):
         debug("read_config(%s) is not a file!", conf_file)
         return d
-    if PYTHON2:
-        mode = "rU"
-    else:
-        mode = "r"
-    with open(conf_file, mode) as f:
+    with open(conf_file, "r") as f:
         lines = []
         no = 0
         for line in f:
@@ -575,9 +558,9 @@ OPTION_TYPES = {
                     "daemon"            : bool,
                     "start-via-proxy"   : bool,
                     "attach"            : bool,
-                    "use-display"       : bool,
+                    "use-display"       : str,
                     "fake-xinerama"     : str,
-                    "resize-display"    : bool,
+                    "resize_display"    : bool,
                     "tray"              : bool,
                     "pulseaudio"        : bool,
                     "dbus-proxy"        : bool,
@@ -623,6 +606,8 @@ OPTION_TYPES = {
                     "compressors"       : list,
                     "packet-encoders"   : list,
                     "key-shortcut"      : list,
+                    "source"            : list,
+                    "source-start"      : list,
                     "start"             : list,
                     "start-child"       : list,
                     "start-after-connect"       : list,
@@ -678,11 +663,8 @@ BIND_OPTIONS = ["bind", "bind-tcp", "bind-udp", "bind-ssl", "bind-ws", "bind-wss
 
 #keep track of the options added since v1,
 #so we can generate command lines that work with older supported versions:
-OPTIONS_ADDED_SINCE_V1 = [
-    "attach", "open-files", "open-url", "pixel-depth",
-    "uid", "gid", "chdir", "min-port", "rfb-upgrade", "bandwidth-limit",
-    "forward-xdg-open", "modal-windows", "bandwidth-detection",
-    "bind-ssh", "ssh-auth", "ssh-upgrade",
+OPTIONS_ADDED_SINCE_V3 = [
+    "source", "source-start",
     ]
 OPTIONS_COMPAT_NAMES = {
     "--compression_level=" : "-z"
@@ -723,9 +705,7 @@ CLIENT_OPTIONS = ["title", "username", "password", "session-name",
                   "env"]
 
 CLIENT_ONLY_OPTIONS = ["username", "swap-keys", "dock-icon",
-                       "tray", "delay-tray", "tray-icon",
-                       "attach",
-                       ]
+                       "tray", "delay-tray", "tray-icon"]
 
 #options that clients can pass to the proxy
 #and which will be forwarded to the new proxy instance process:
@@ -743,7 +723,8 @@ PROXY_START_OVERRIDABLE_OPTIONS = [
     "input-method",
     "microphone", "speaker", "sound-source", "pulseaudio",
     "idle-timeout", "server-idle-timeout",
-    "use-display", "fake-xinerama", "resize-display", "dpi", "pixel-depth",
+    "use-display",
+    "fake-xinerama", "resize_display", "dpi", "pixel-depth",
     "readonly", "keyboard-sync", "cursors", "bell", "notifications", "xsettings",
     "system-tray", "sharing", "lock", "windows", "webcam", "html",
     "terminate-children", "exit-with-children", "exit-with-client",
@@ -755,7 +736,7 @@ PROXY_START_OVERRIDABLE_OPTIONS = [
     "auth", "vsock-auth", "tcp-auth", "udp-auth", "ws-auth", "wss-auth", "ssl-auth", "ssh-auth", "rfb-auth",
     "bind", "bind-vsock", "bind-tcp", "bind-udp", "bind-ssl", "bind-ws", "bind-wss", "bind-ssh", "bind-rfb",
     "rfb-upgrade", "bandwidth-limit",
-    "start", "start-child",
+    "start", "start-child", "source", "source-start",
     "start-after-connect", "start-child-after-connect",
     "start-on-connect", "start-child-on-connect",
     "start-on-last-client-exit", "start-child-on-last-client-exit",
@@ -800,15 +781,11 @@ def get_default_key_shortcuts():
 def get_default_systemd_run():
     if WIN32 or OSX:
         return "no"
-    #don't use systemd-run on CentOS / RedHat
-    #(it causes failures with "Failed to create bus connection: No such file or directory")
-    if is_CentOS() or is_RedHat():
-        return "no"
-    if is_Fedora():
-        return "no"
     #systemd-run was previously broken in Fedora 26:
     #https://github.com/systemd/systemd/issues/3388
     #but with newer kernels, it is working again..
+    #now that we test it before using it,
+    #it should be safe to leave it on auto:
     return "auto"
 
 def get_default_pulseaudio_command():
@@ -824,11 +801,9 @@ def get_default_pulseaudio_command():
         "--load=module-x11-publish",
         "--log-level=2", "--log-target=stderr",
         ]
-    #we don't enable memfd on Ubuntu 16.04,
-    #we just don't disable it (because the option does not exist!):
     from xpra.util import envbool
     MEMFD = envbool("XPRA_PULSEAUDIO_MEMFD", False)
-    if not MEMFD and (not is_Ubuntu() or getUbuntuVersion()>=(18, 4)):
+    if not MEMFD:
         cmd.append("--enable-memfd=no")
     return cmd
 
@@ -871,24 +846,23 @@ def get_defaults():
     for conf_dir in conf_dirs:
         if conf_dir and os.path.exists(conf_dir):
             break
-    xvfb = detect_xvfb_command(conf_dir, bin_dir)
+    xvfb = detect_xvfb_command(conf_dir, bin_dir, warn=None)
     if WIN32:
         bind_dirs = ["Main"]
     else:
         bind_dirs = ["auto"]
 
     ssl_protocol = "TLSv1_2"
-    if sys.version_info<(2, 7, 9):
-        ssl_protocol = "SSLv23"
 
-    if POSIX and not OSX and not (is_Debian() or is_Ubuntu()):
-        fake_xinerama = "auto"
+    if POSIX and not OSX:
+        from xpra.x11.fakeXinerama import find_libfakeXinerama
+        fake_xinerama = find_libfakeXinerama() or "no"
     else:
         fake_xinerama = "no"
 
     GLOBAL_DEFAULTS = {
                     "encoding"          : "auto",
-                    "title"             : "@title@ on @client-machine@",
+                    "title"             : "@title@ on @hostname@",
                     "username"          : username,
                     "password"          : "",
                     "wm-name"           : DEFAULT_NET_WM_NAME,
@@ -966,7 +940,7 @@ def get_defaults():
                     "ssl-server-verify-mode"   : "required",
                     "ssl-verify-flags"  : "X509_STRICT",
                     "ssl-check-hostname": False,
-                    "ssl-server-hostname": "",
+                    "ssl-server-hostname": "localhost",
                     "ssl-options"       : "ALL,NO_COMPRESSION",
                     "quality"           : 0,
                     "min-quality"       : 30,
@@ -987,7 +961,7 @@ def get_defaults():
                     "daemon"            : CAN_DAEMONIZE,
                     "start-via-proxy"   : False,
                     "attach"            : None,
-                    "use-display"       : False,
+                    "use-display"       : "no",
                     "fake-xinerama"     : fake_xinerama,
                     "resize-display"    : not OSX and not WIN32,
                     "tray"              : True,
@@ -995,7 +969,7 @@ def get_defaults():
                     "dbus-proxy"        : not OSX and not WIN32,
                     "mmap"              : "yes",
                     "mmap-group"        : "auto",
-                    "speaker"           : ["disabled", "on"][has_sound_support()],
+                    "speaker"           : ["disabled", "on"][has_sound_support() and not is_arm()],
                     "microphone"        : ["disabled", "off"][has_sound_support()],
                     "video-scaling"     : "auto",
                     "readonly"          : False,
@@ -1025,7 +999,7 @@ def get_defaults():
                     "desktop-fullscreen": False,
                     "global-menus"      : True,
                     "forward-xdg-open"  : True,
-                    "modal-windows"     : True,
+                    "modal-windows"     : False,
                     "bandwidth-detection" : True,
                     "ssh-upgrade"       : True,
                     "pulseaudio-configure-commands"  : [" ".join(x) for x in DEFAULT_PULSEAUDIO_CONFIGURE_COMMANDS],
@@ -1060,6 +1034,8 @@ def get_defaults():
                     "ssh-auth"          : [],
                     "rfb-auth"          : [],
                     "password-file"     : [],
+                    "source"            : [],
+                    "source-start"      : [],
                     "start"             : [],
                     "start-child"       : [],
                     "start-after-connect"       : [],
@@ -1163,13 +1139,10 @@ def parse_with_unit(numtype, v, subunit="bps", min_value=250000):
             assert f>=min_value, "value is too low"
         return int(f)
     except Exception as e:
-        raise InitException("invalid value for %s '%s': %s" % (numtype, v, e))
+        raise InitException("invalid value for %s '%s': %s" % (numtype, v, e)) from None
 
 
-def validate_config(d=None, discard=NO_FILE_OPTIONS, extras_types=None, extras_validation=None):
-    return do_validate_config(d or {}, discard, extras_types or {}, extras_validation or {})
-
-def do_validate_config(d, discard, extras_types, extras_validation):
+def validate_config(d={}, discard=NO_FILE_OPTIONS, extras_types={}, extras_validation={}):
     """
         Validates all the options given in a dict with fields as keys and
         strings or arrays of strings as values.
@@ -1227,20 +1200,16 @@ def do_validate_config(d, discard, extras_types, extras_validation):
     return nd
 
 
-def make_defaults_struct(extras_defaults=None, extras_types=None, extras_validation=None, username="", uid=getuid(), gid=getgid()):
-    return do_make_defaults_struct(extras_defaults or {}, extras_types or {}, extras_validation or {}, username, uid, gid)
-
-def do_make_defaults_struct(extras_defaults, extras_types, extras_validation, username, uid, gid):
+def make_defaults_struct(extras_defaults={}, extras_types={}, extras_validation={}, username="", uid=getuid(), gid=getgid()):
     #populate config with default values:
     if not username and uid:
         username = get_username_for_uid(uid)
     defaults = read_xpra_defaults(username, uid, gid)
     return dict_to_validated_config(defaults, extras_defaults, extras_types, extras_validation)
 
-def dict_to_validated_config(d, extras_defaults=None, extras_types=None, extras_validation=None):
+def dict_to_validated_config(d={}, extras_defaults={}, extras_types={}, extras_validation={}):
     options = get_defaults().copy()
-    if extras_defaults:
-        options.update(extras_defaults)
+    options.update(extras_defaults)
     #parse config:
     validated = validate_config(d, extras_types=extras_types, extras_validation=extras_validation)
     options.update(validated)
@@ -1256,7 +1225,7 @@ def dict_to_config(options):
     return config
 
 
-class XpraConfig(object):
+class XpraConfig:
     def __repr__(self):
         return "XpraConfig(%s)" % self.__dict__
 
@@ -1439,9 +1408,11 @@ def abs_paths(options):
                 continue
             setattr(options, f, os.path.abspath(v))
 
-def fixup_options(options):
+
+def fixup_options(options, skip_encodings=False):
+    if not skip_encodings:
+        fixup_encodings(options)
     fixup_pings(options)
-    fixup_encodings(options)
     fixup_compression(options)
     fixup_packetencoding(options)
     fixup_video_all_or_none(options)
