@@ -77,6 +77,7 @@ LEGACY_SALT_DIGEST = envbool("XPRA_LEGACY_SALT_DIGEST", True)
 CHALLENGE_TIMEOUT = envint("XPRA_CHALLENGE_TIMEOUT", 120)
 SYSCONFIG = envbool("XPRA_SYSCONFIG", True)
 
+ENCRYPTED_SOCKET_TYPES = os.environ.get("XPRA_ENCRYPTED_SOCKET_TYPES", "tcp,ws")
 
 HTTP_UNSUPORTED = b"""HTTP/1.1 400 Bad request syntax or unsupported method
 
@@ -1261,7 +1262,7 @@ class ServerCore:
         protocol.authenticators = ()
         protocol.encryption = None
         protocol.keyfile = None
-        if socktype=="tcp":
+        if socktype in ENCRYPTED_SOCKET_TYPES:
             #special case for legacy encryption code:
             protocol.encryption = socket_options.get("encryption", self.tcp_encryption)
             protocol.keyfile = socket_options.get("encryption-keyfile", self.tcp_encryption_keyfile)
@@ -1696,9 +1697,9 @@ class ServerCore:
         if not username:
             import getpass
             username = getpass.getuser()
+        conn = proto._conn
         #authenticator:
         if not proto.authenticators:
-            conn = proto._conn
             socktype = conn.socktype_wrapped
             try:
                 proto.authenticators = self.make_authenticators(socktype, username, conn)
@@ -1740,7 +1741,7 @@ class ServerCore:
             auth_caps = new_cipher_caps(proto, cipher, encryption_key, padding_options)
             authlog("server cipher=%s", auth_caps)
         else:
-            if proto.encryption:
+            if proto.encryption and conn.socktype in ENCRYPTED_SOCKET_TYPES:
                 authlog("client does not provide encryption tokens")
                 auth_failed("missing encryption tokens")
                 return
