@@ -1030,7 +1030,7 @@ class XpraServer(GObject.GObject, X11ServerBase):
                         #try to ensure this won't trigger a resizing loop:
                         counter = max(0, resize_counter-1)
                         for s in self._server_sources.values():
-                            if s!=ss and isinstance(ss, WindowsMixin):
+                            if s!=ss and isinstance(s, WindowsMixin):
                                 s.resize_window(wid, window, aw, ah, resize_counter=counter)
                     damage |= owx!=ax or owy!=ay or resized
             if not shown and not skip_geometry:
